// @flow
import type { ItemQuery, SupportedStores, RakutenItem, Item } from '../types/Item'
import uuid from 'uuid/v4'
import d from 'debug'

const debug = d('data-access:item')

class ItemError extends Error {
  code: number
  message: string

  constructor (options: { message: string, code: number }) {
    super(options.message)
    this.code = options.code
    this.message = options.message
  }
}

export const list = (db: any) => (store: SupportedStores) => async (query: ItemQuery) => {
  try {
    const storeCol = db.collection(store)

    const items: Array<Item> = await storeCol.find(query).toArray()
    return items
  } catch (e) {
    debug(e)
    throw (e)
  }
}

export const get = (db: any) => (store: SupportedStores) => async (query: ItemQuery) => {
  try {
    const storeCol = db.collection(store)

    const item: Item = await storeCol.findOne(query)
    if (!item) {
      throw new ItemError({ message: 'ItemNotFound', code: 404 })
    }
    return item
  } catch (e) {
    debug(e)
    throw e
  }
}

export const save = (db: any) => async (item: RakutenItem) => {
  const { store, itemId } = item

  try {
    const storeCol = db.collection(store)
    const date = new Date()

    // Add the date to the offer
    const offers = item.offers.map(offer => Object.assign({}, offer, { date }))

    const [lowestNewOffer] = offers.sort((x, y) => x.price - y.price)

    const { value }: { value: Item } = await storeCol.findOneAndUpdate(
      { itemId },
      {
        $set: Object.assign({}, item, { offers, updatedAt: date }),
        $setOnInsert: { offerHistory: [lowestNewOffer], createdAt: date, groupId: item.upc || uuid() }
      },
      { upsert: true, returnOriginal: false }
    )

    const { offerHistory = [] } = value
    const [lastOfferHistory] = offerHistory.sort((x, y) => y.date - x.date)

    if (offerHistory.length === 0 || lastOfferHistory.price !== lowestNewOffer.price) {
      const { value: newValue }: { value: Item } = await storeCol.findOneAndUpdate(
        { itemId },
        {
          $push: { offerHistory: lowestNewOffer }
        },
        { returnOriginal: false }
      )

      return newValue
    }

    return value
  } catch (e) {
    throw e
  }
}
