// @flow
import type { SupportedStores, Item } from '../types/Item'
import type { UserItem, UserItemDoc, UserItemOffer, UserItemInput, UserItemQuery } from '../types/UserItem'
import type { User } from '../types/User'

import { get as getUser } from './User'
import d from 'debug'
const debug = d('data-access:userItem')

const STORES: Array<SupportedStores> = ['amazon', 'bestbuy', 'walmart', 'target', 'gamestop']

const buildUserItem = (db: any) => async (userItemDoc: UserItemDoc): Promise<UserItem> => {
  const { userId, goal: price, updatedAt, result, groupId } = userItemDoc
  const items: Array<Item> = await Promise.all(STORES.map(store => db.collection(store).findOne({ groupId }))).then(s => s.filter(_s => !!_s))

  if (!items || items.length === 0) {
    throw new Error(`No Items Found with groupId: ${groupId}`)
  }

  // Get Global Value
  const [{ title, msrp }] = items
  const goal = { price, result }
  const offers: Array<UserItemOffer> = items.map(i => {
    const { url, itemId, store, updatedAt, offers } = i
    const offer = offers.sort((x, y) => x.price - y.price).pop()
    return Object.assign({}, offer, { url, itemId, store, updatedAt })
  })

  // Get lowest Priced Offer
  const offer = offers.sort((x, y) => x.price - y.price).pop()

  return {
    goal,
    userId,
    groupId,
    title,
    msrp,
    updatedAt,
    offer,
    offers
  }
}

export const add = (db: any) => async (input: UserItemInput) => {
  const { userId, groupId } = input
  const now = new Date()

  try {
    const user: User = await getUser(db)(userId)

    if (!user) {
      throw new Error('Invalid User')
    }

    const { value }: { value: UserItemDoc } = await db.collection('userItems').findOneAndUpdate(
      { userId, groupId },
      {
        $set: Object.assign({}, input, { updatedAt: now }),
        $setOnInsert: { createdAt: new Date(), result: false }
      },
      { returnOriginal: false, upsert: true }
    )

    return buildUserItem(db)(value)
  } catch (e) {
    debug(e)
    throw e
  }
}

export const update = (db: any) => async (input: UserItemInput) => {
  const { userId, groupId, goal } = input

  try {
    const { value }: { value: UserItemDoc } = await db.collection('userItems').findOneAndUpdate(
      { userId, groupId },
      {
        $set: { goal }
      },
      { returnOriginal: false }
    )

    return buildUserItem(db)(value)
  } catch (e) {
    throw e
  }
}

export const get = (db: any) => async (query: UserItemQuery) => {
  const { userId, groupId } = query

  try {
    const _userItem: UserItemDoc = await db.collection('userItems').findOne({ userId, groupId }, { _id: 0 })

    return buildUserItem(db)(_userItem)
  } catch (e) {
    debug(e)
    throw e
  }
}

export const list = (db: any) => async (userId: string) => {
  try {
    const userItemDocs = await db.collection('userItems').find({ userId }).toArray()

    return Promise.all(
      userItemDocs.map(buildUserItem(db))
    )
  } catch (e) {
    debug(e)
    throw e
  }
}

export const remove = (db: any) => async ({ userId, groupId }: { userId: string, groupId: string }) => {
  try {
    const { result } = await db.collection('userItems').remove({ userId, groupId })
    if (result.n === 0) {
      throw new Error('Item Removal Error')
    }
    return result.n === 1
  } catch (e) {
    debug(e)
    throw e
  }
}
