const { describe, it, after, before, afterEach, beforeEach } = require('mocha')
const { expect } = require('chai')
const {
  save: saveItem,
  get: getItem,
  list: listItems
} = require('../../src/lib/Item')
const items = require('./items.json')

describe('Items', function () {
  const {
    DB_DATABASE: dbName,
    DB_HOST: host,
    DB_PORT: port
  } = process.env

  let client
  let db

  before('Connect', function () {
    const { MongoClient } = require('mongodb')
    const url = `mongodb://${host}:${port}`

    return MongoClient.connect(url, { poolSize: 10 })
      .then(_client => {
        client = _client
        db = client.db(dbName)
      })
  })

  after('Disconnect', function () {
    db
      .dropDatabase(dbName)
      .catch(e => e)
      .then(() => {
        client.close()
      })
  })

  afterEach(() => db.dropCollection('bestbuy'))

  describe('Save', function () {
    it('Saves an Item', function () {
      const item = Object.assign({}, items[0])
      return saveItem(db)(item)
        .then(value => {
          expect(value._id).to.be.ok
          expect(value.groupId).to.equal(value.upc)
          expect(value.offerHistory).to.have.length(1)
        })
    })

    it('Saves an Item without a upc', function () {
      const item = Object.assign({}, items[0])
      delete item.upc

      return saveItem(db)(item)
        .then(value => {
          expect(value._id).to.be.ok
          expect(value.groupId).to.be.ok
          expect(value.offerHistory).to.have.length(1)
        })
    })

    it('Updates Items and tracks OfferHistory', async function () {
      const item = Object.assign(items[0])
      const [offer] = items[0].offers

      const item1 = await saveItem(db)(item)
      const item2 = await saveItem(db)(Object.assign({}, item, {
        offers: [Object.assign({}, offer, { price: 31.00 })]
      }))
      const item3 = await saveItem(db)(item)
      const item4 = await saveItem(db)(item)

      expect(item1._id).to.be.ok
      expect(item1.offerHistory).to.have.length(1)
      expect(item1.offerHistory[0].price).to.equal(32.99)

      expect(item2._id).to.be.ok
      expect(item2.offerHistory).to.have.length(2)
      expect(item2.offerHistory[0].price).to.equal(32.99)
      expect(item2.offerHistory[1].price).to.equal(31.00)

      expect(item3._id).to.be.ok
      expect(item3.offerHistory).to.have.length(3)
      expect(item3.offerHistory[0].price).to.equal(32.99)
      expect(item3.offerHistory[1].price).to.equal(31.00)
      expect(item3.offerHistory[2].price).to.equal(32.99)

      expect(item4._id).to.be.ok
      expect(item4.offerHistory).to.have.length(3)
    })
  })

  describe('Get', function () {
    const item = Object.assign({}, items[0])

    beforeEach(function () {
      return saveItem(db)(item)
    })

    it('should return using itemId', async function () {
      const { store, itemId } = item
      const item1 = await getItem(db)(store)({ itemId })

      expect(item1._id).to.be.ok
      expect(item1.itemId).to.equal(itemId)
      expect(item1.title).to.equal(item.title)
    })

    it('should return using upc', async function () {
      const { store, upc } = item
      const item1 = await getItem(db)(store)({ upc })

      expect(item1._id).to.be.ok
      expect(item1.title).to.equal(item.title)
      expect(item1.upc).to.equal(upc)
    })

    it('should throw an exception if item is not found', function () {
      const { store } = item

      return getItem(db)(store)({ itemId: '0000' })
        .catch(e => {
          expect(e.code).to.equal(404)
        })
    })
  })

  describe('List', function () {
    const _items = [
      Object.assign({}, items[0]),
      Object.assign({}, items[1]),
      Object.assign({}, items[2])
    ]
    const [item] = _items

    beforeEach(function () {
      return Promise.all(_items.map(saveItem(db)))
    })

    it('should return using category', async function () {
      const { store, category } = item
      const [item1, item2, item3] = await listItems(db)(store)({ category })

      expect(item1._id).to.be.ok
      expect(item1.category).to.equal(category)
      expect(item1.title).to.equal(item.title)

      expect(item2._id).to.be.ok
      expect(item2.category).to.equal(category)

      expect(item3).to.not.be.ok
    })

    it('should return using upc', async function () {
      const { store, upc } = item
      const item1 = await getItem(db)(store)({ upc })

      expect(item1._id).to.be.ok
      expect(item1.title).to.equal(item.title)
      expect(item1.upc).to.equal(upc)
    })
  })
})
