const { describe, it, after, before, afterEach, beforeEach } = require('mocha')
const { expect } = require('chai')
const {
  add: addUserItem,
  update: updateUserItem,
  get: getUserItem,
  list: getAllUserItems,
  remove: removeUserItem
} = require('../../src/lib/UserItem')

const { save: saveUser } = require('../../src/lib/User')
const { save: saveItem } = require('../../src/lib/Item')

const load = async db => {
  const users = require('./users.json')
  await Promise.all(users.map(saveUser(db)))
  // Save as BestBuy
  await Promise.all(require('./items.json').map(saveItem(db)))

  // Save as Amazon
  await Promise.all(require('./items.json').map(item => {
    return saveItem(db)(Object.assign({}, item, {
      store: 'amazon',
      itemId: Math.round(Math.random() * 10000)
    }))
  }))

  // Save as Walmart
  await Promise.all(require('./items.json').map(item => {
    return saveItem(db)(Object.assign({}, item, {
      store: 'walmart',
      itemId: Math.round(Math.random() * 10000)
    }))
  }))

  const bbItems = await db.collection('bestbuy').find({}).toArray()
  const azItems = await db.collection('amazon').find({}).toArray()
  const wlItems = await db.collection('walmart').find({}).toArray()
  
  expect(bbItems).to.have.length(50)
  expect(azItems).to.have.length(50)
  expect(wlItems).to.have.length(50)
}

describe('UserItems', function () {
  const {
    DB_DATABASE: dbName,
    DB_HOST: host,
    DB_PORT: port
  } = process.env

  let client
  let db

  before('Connect', function () {
    const { MongoClient } = require('mongodb')
    const url = `mongodb://${host}:${port}`

    return MongoClient.connect(url, { poolSize: 10 })
      .then(_client => {
        client = _client
        db = client.db(dbName)
      })
  })

  after('Disconnect', function () {
    return db
      .dropDatabase(dbName)
      .catch(e => e)
      .then(() => {
        client.close()
      })
  })

  describe('Add', function () {
    before('Add User and Item', () => load(db))

    after(() => db.dropDatabase(dbName))
    afterEach(() => db.collection('userItems').drop())

    it('should save a add a userItem', async function () {
      const [item] = require('./items.json')
      const [user] = require('./users.json')

      const { id: userId } = user
      const { upc: groupId } = item
      const userItem = { userId, groupId, goal: 30 }
      const _userItem = await addUserItem(db)(userItem)

      expect(_userItem.userId).to.equal(userId)
      expect(_userItem.title).to.equal(item.title)
      expect(_userItem.offers).to.have.length(2)
      expect(_userItem.groupId).to.equal(item.upc)
      expect(_userItem.goal.price).to.equal(30)

      const { offer } = _userItem
      expect(offer.available).to.equal(true)
    })

    it('should update the goal', async function () {
      const [item] = require('./items.json')
      const [user] = require('./users.json')

      const { id: userId } = user
      const { upc: groupId } = item
      await addUserItem(db)({ userId, groupId, goal: 30 })

      const _userItem = await updateUserItem(db)({ userId, groupId, goal: 40 })

      expect(_userItem.userId).to.equal(userId)
      expect(_userItem.title).to.equal(item.title)
      expect(_userItem.offers).to.have.length(2)
      expect(_userItem.groupId).to.equal(item.upc)
      expect(_userItem.goal.price).to.equal(40)
    })

    it('should retrieve a userItem', async function () {
      const [item] = require('./items.json')
      const [user] = require('./users.json')

      const { id: userId } = user
      const { upc: groupId } = item
      await addUserItem(db)({ userId, groupId, goal: 30 })

      const _userItem = await getUserItem(db)({ userId, groupId })

      expect(_userItem.userId).to.equal(userId)
      expect(_userItem.title).to.equal(item.title)
      expect(_userItem.offers).to.have.length(2)
      expect(_userItem.groupId).to.equal(item.upc)
      expect(_userItem.goal.price).to.equal(30)

      const { offer } = _userItem
      expect(offer.available).to.equal(true)
    })

    it('should list all userItems', async function () {
      const [item, item2] = require('./items.json')
      const [user] = require('./users.json')

      const { id: userId } = user
      await addUserItem(db)({ userId, groupId: item.upc, goal: 30 })
      await addUserItem(db)({ userId, groupId: item2.upc, goal: 30 })

      const userItems = await getAllUserItems(db)(userId)

      expect(userItems).to.have.length(2)

      userItems.forEach(i => {
        expect(i.goal.price).to.equal(30)
        expect(i.offers).to.have.length(2)
      })
    })

    it('should delete a userItem', async function () {
      const [item, item2] = require('./items.json')
      const [user, user2] = require('./users.json')

      const { id: userId } = user
      const { id: userId2 } = user2

      await addUserItem(db)({ userId, groupId: item.upc, goal: 30 })
      await addUserItem(db)({ userId, groupId: item2.upc, goal: 30 })

      await addUserItem(db)({ userId: userId2, groupId: item.upc, goal: 30 })
      await addUserItem(db)({ userId: userId2, groupId: item2.upc, goal: 30 })

      await removeUserItem(db)({ userId, groupId: item.upc })

      const userList = await getAllUserItems(db)(userId)
      const userList2 = await getAllUserItems(db)(userId2)

      expect(userList).to.have.length(1)
      expect(userList.map(userItem => userItem.groupId)).to.not.include(item.upc)

      expect(userList2).to.have.length(2)
      expect(userList2.map(userItem => userItem.groupId)).to.include(item.upc)
    })

    it('should throw an exception if the userItem does not exist', async function () {
      const [item, item2] = require('./items.json')
      const [user, user2] = require('./users.json')

      const { id: userId } = user
      const { id: userId2 } = user2

      await addUserItem(db)({ userId, groupId: item.upc, goal: 30 })
      await addUserItem(db)({ userId, groupId: item2.upc, goal: 30 })

      await addUserItem(db)({ userId: userId2, groupId: item.upc, goal: 30 })
      await addUserItem(db)({ userId: userId2, groupId: item2.upc, goal: 30 })

      let e

      try {
        await removeUserItem(db)({ userId, groupId: '000000' })
      } catch (_e) {
        e = _e
      }

      expect(e).to.be.ok

      const userList = await getAllUserItems(db)(userId)
      const userList2 = await getAllUserItems(db)(userId2)

      expect(userList).to.have.length(2)
      expect(userList.map(userItem => userItem.groupId)).to.include(item.upc)

      expect(userList2).to.have.length(2)
      expect(userList2.map(userItem => userItem.groupId)).to.include(item.upc)
    })
  })

  // describe('Engine', function () {
  //   let users
  //   let items

  //   before('Add Users and Items', async function () {
  //     const _users = require('./users.json')
  //     const _items = require('./items.json')

  //     users = await Promise.all(_users.map(saveUser(db)))
  //     items = await Promise.all(_items.map(saveItem(db)))

  //     // Add Items to both Users
  //     return Promise.all(
  //       items.map(item => Promise.all(users.map(user => {
  //         const { _id: _itemId, itemId } = item
  //         const { _id: _userId } = user

  //         const userItem = Object.assign({}, { goal: Math.round(Math.random() * 100), store: 'bestbuy' }, { _itemId, _userId, itemId })

  //         return saveUserItem(db)(userItem)
  //       })))
  //     )
  //   })

  //   it('should retrieve user items grouped for engine', async function () {
  //     const items = await engine(db)('bestbuy')

  //     expect(items).to.have.length(50)

  //     items.forEach(item => {
  //       expect(item.itemId).to.be.ok
  //       expect(item.userItems).to.have.length(2)
  //     })
  //   })
  // })
})
